package com.mortals.xhx.common.plugin;

import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.exception.RenderException;
import com.deepoove.poi.policy.RenderPolicy;
import com.deepoove.poi.template.ElementTemplate;
import com.deepoove.poi.template.run.RunTemplate;
import com.deepoove.poi.util.TableTools;
import org.apache.poi.xwpf.usermodel.*;

import java.util.Iterator;
import java.util.List;

/**
 * 单行数据填充，不需要定义列元素
 *
 * @author:
 * @date: 2022/11/30 13:34
 */
public class LoopSingleRowTableRenderPolicy implements RenderPolicy {


	@Override
	public void render(ElementTemplate eleTemplate, Object data, XWPFTemplate template) {
		RunTemplate runTemplate = (RunTemplate) eleTemplate;
		XWPFRun run = runTemplate.getRun();
		try {
			if (!TableTools.isInsideTable(run)) {
				throw new IllegalStateException(
					"The template tag " + runTemplate.getSource() + " must be inside a table");
			}
			//获取设置标志的单元
			XWPFTableCell tagCell = (XWPFTableCell) ((XWPFParagraph) run.getParent()).getBody();
			XWPFTable table = tagCell.getTableRow().getTable();
			XWPFTableRow templateRow=tagCell.getTableRow();
			//重置
			run.setText("", 0);
			//获取模板列index
			int templateColIndex = getTemplateColIndex(tagCell) + 1;
			if (null != data && data instanceof Iterable) {
				//data数据渲染
				Iterator<?> iterator = ((Iterable<?>) data).iterator();
				boolean hasNext = iterator.hasNext();
				while (hasNext) {
					Object root = iterator.next();
					hasNext = iterator.hasNext();
					//填充当前列元素
					setTableRowCol(templateRow, root.toString(), templateColIndex);
					templateColIndex++;

				}
			}

			afterloop(table, data);
		} catch (Exception e) {
			e.printStackTrace();
			throw new RenderException("HackLoopTable for " + eleTemplate + "error: " + e.getMessage(), e);
		}
	}
	private int getTemplateColIndex(XWPFTableCell tagCell) {
		String text = tagCell.getText();
		int col = -1;
		XWPFTableRow tagRow = tagCell.getTableRow();
		List<XWPFTableCell> tableCells = tagRow.getTableCells();

		for (int i = 0; i < tableCells.size(); i++) {
			if (tableCells.get(i).getText().equalsIgnoreCase(text)) {
				return i;
			}
		}
		return col;
	}

	protected void afterloop(XWPFTable table, Object data) {
	}

	private void setTableRowCol(XWPFTableRow templateRow, String str, int pos) {
		int limit = templateRow.getTableCells().size();
		if (pos > limit) {
			return;
		}
		XWPFTableCell cell = templateRow.getCell(pos);
		if (cell != null) {
			cell.setText(str);
		}
	}

}
