package com.mortals.xhx.module.matter.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.text.StrSplitter;
import cn.hutool.core.text.finder.LengthFinder;
import cn.hutool.core.text.split.SplitIter;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.data.PictureRenderData;
import com.deepoove.poi.data.PictureType;
import com.deepoove.poi.data.Pictures;
import com.deepoove.poi.plugin.table.MultipleRowTableRenderPolicy;
import com.deepoove.poi.template.MetaTemplate;
import com.deepoove.poi.util.RegexUtils;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.common.Rest;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.util.DataUtil;
import com.mortals.framework.util.StringUtils;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.base.system.upload.service.UploadService;
import com.mortals.xhx.common.code.ComponentEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.formdesign.*;
import com.mortals.xhx.common.key.Constant;
import com.mortals.xhx.common.key.ParamKey;
import com.mortals.xhx.common.pdu.Col;
import com.mortals.xhx.common.pdu.gen.component.ComponentCons;
import com.mortals.xhx.common.plugin.LoopSingleRowTableRenderPolicy;
import com.mortals.xhx.common.utils.ExportDocUtil;
import com.mortals.xhx.common.utils.WordUtil;
import com.mortals.xhx.module.matter.dao.MatterDao;
import com.mortals.xhx.module.matter.model.MatterDatumQuery;
import com.mortals.xhx.module.matter.model.MatterEntity;
import com.mortals.xhx.module.matter.model.MatterQuery;
import com.mortals.xhx.module.matter.service.MatterService;
import com.mortals.xhx.module.pubdatum.model.PubdatumEntity;
import com.mortals.xhx.module.pubdatum.model.PubdatumQuery;
import com.mortals.xhx.module.pubdatum.service.PubdatumService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.io.FilenameUtils;
import org.apache.http.entity.ContentType;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.matter.dao.MatterDatumDao;
import com.mortals.xhx.module.matter.model.MatterDatumEntity;
import com.mortals.xhx.module.matter.service.MatterDatumService;
import org.springframework.util.ObjectUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.util.*;
import java.util.stream.Collectors;

/**
 * MatterDatumService
 * 事项申请材料 service实现
 *
 * @author zxfei
 * @date 2022-09-27
 */
@Service("matterDatumService")
public class MatterDatumServiceImpl extends AbstractCRUDServiceImpl<MatterDatumDao, MatterDatumEntity, Long> implements MatterDatumService {

    @Value("${upload.path}")
    private String filePath;
    @Value("${upload.url:http://localhost:17215/fsm/file/commonupload?prePath=/file/uploadfile}")
    private String uploadUrl;
    @Autowired
    private UploadService uploadService;
    @Autowired
    private MatterService matterService;

    @Autowired
    private PubdatumService pubdatumService;

    @Override
    protected MatterDatumEntity findBefore(MatterDatumEntity params, PageInfo pageInfo, Context context) throws AppException {
        if (com.mortals.xhx.common.utils.StringUtils.isNotEmpty(params.getMaterialName())) {
            params.setMaterialName("%".concat(params.getMaterialName()).concat("%"));
        }
        return params;
    }

    @Override
    protected void saveAfter(MatterDatumEntity entity, Context context) throws AppException {
        statDatumCount(entity, context);
        addPubdatum(entity, context);
        super.saveAfter(entity, context);
    }

    @Override
    protected void updateBefore(MatterDatumEntity entity, Context context) throws AppException {
        super.updateBefore(entity, context);
    }

    @Override
    protected void updateAfter(MatterDatumEntity entity, Context context) throws AppException {
        addPubdatum(entity, context);
        super.updateAfter(entity, context);
    }


    private void addPubdatum(MatterDatumEntity entity, Context context) {
        if (!ObjectUtils.isEmpty(entity.getPub()) && entity.getPub() == YesNoEnum.YES.getValue()) {
            int count = pubdatumService.count(new PubdatumQuery().materialName(entity.getMaterialName()), context);
            if (count > 0) {
                throw new AppException("当前公共库已存在当前材料名！");
            }

            //添加到公共库
            PubdatumEntity pubdatumEntity = new PubdatumEntity();
            pubdatumEntity.initAttrValue();
            BeanUtils.copyProperties(entity, pubdatumEntity, new String[]{"id"});

            pubdatumEntity.setCreateTime(new Date());
            pubdatumEntity.setCreateUserId(getContextUserId(context));
            pubdatumService.save(pubdatumEntity, context);
        }
    }

    private void statDatumCount(MatterDatumEntity matterDatumEntity, Context context) {
        int count = this.count(new MatterDatumQuery().matterId(matterDatumEntity.getMatterId()), context);
        MatterEntity matterEntity = matterService.get(matterDatumEntity.getMatterId(), context);
        if (!ObjectUtils.isEmpty(matterEntity)) {
            matterEntity.setDatumCount(count);
            matterService.update(matterEntity, context);
        }
    }

    @Override
    protected void saveBefore(MatterDatumEntity entity, Context context) throws AppException {
        if (StringUtils.isEmpty(entity.getFileUrl())) {
            throw new AppException("填单附件不能为空");
        }
        if (StringUtils.isEmpty(entity.getSamplePath())) {
            throw new AppException("样表地址不能为空");
        }
        if (StringUtils.isEmpty(entity.getTemplatePath())) {
            throw new AppException("样表模板地址不能为空");
        }
        //校验材料名是否重复
        int count = this.count(new MatterDatumQuery().materialName(entity.getMaterialName()), context);
        if (count > 0) {
            throw new AppException("当前材料名称已存在！");
        }

        //生成样表预览图片
        String rootPath = this.filePath.endsWith("/") ? this.filePath : this.filePath + "/";
        String samplePath = rootPath + entity.getSamplePath();
        String prePath = "file/preview/";
        String filePath = rootPath + prePath;
        File pathDir = new File(filePath);
        if (!pathDir.exists()) {
            pathDir.mkdirs();
        }

        String newName = RandomUtil.randomNumbers(12) + ".jpg";
        String filePathAll = filePath + newName;
        WordUtil.convertWordToJPEG(samplePath, filePathAll);

        entity.setPreViewPath(prePath + newName);
        //校验模板变量正确性
        if (!ObjectUtils.isEmpty(entity.getTemplatePath())) {
            parseDocxToJson(entity);
        }

        super.saveBefore(entity, context);
    }

    @Override
    public int remove(Long id, Context context) throws AppException {
        this.removeBefore(id, context);
        MatterDatumEntity matterDatumEntity = this.get(id, context);
        int iRet = this.dao.delete(id);
        this.removeAfter(id, context, iRet);
        statDatumCount(matterDatumEntity, context);
        return iRet;
    }

    @Override
    public int remove(Long[] ids, Context context) throws AppException {
        this.removeBefore(ids, context);
        if (ids == null || ids.length == 0) return 0;
        MatterDatumEntity matterDatumEntity = this.get(ids[0], context);
        int iRet = this.dao.delete(ids);
        this.removeAfter(ids, context, iRet);
        statDatumCount(matterDatumEntity, context);
        return iRet;
    }

    private void parseDocxToJson(MatterDatumEntity entity) {
        String rootPath = this.filePath.endsWith("/") ? this.filePath : this.filePath + "/";
        String filepath = rootPath + entity.getTemplatePath();
        ConfigureBuilder builder = Configure.builder();

        builder.setValidErrorHandler(new Configure.DiscardHandler());
        builder.useDefaultEL(false);
        builder.buildGrammerRegex(RegexUtils.createGeneral("{{", "}}"));
        //builder.buildGramer("[","]");
        XWPFTemplate template = XWPFTemplate.compile(filepath, builder.build());
        FormDesignRoot root = new FormDesignRoot();
        FormDesignConfig config = new FormDesignConfig();
        config.initAttribute();
        root.setConfig(config);
        List<ListItem> collect = new ArrayList<>();
        HashSet<String> hs = new HashSet<>(); //去除重复标签
        for (MetaTemplate item : template.getElementTemplates()) {
            if (hs.contains(item.variable())) {
                continue;
            } else {
                hs.add(item.variable());
            }
            //识别简单输入
            String val = StrUtil.strip(item.variable(), "{{", "}}");
            List<String> keys = StrSplitter.split(val, "_", true, true);
            //切分分隔符后动态 创建组件  i_1_姓名（第一个代表类型，第二个如果是多个累加，第三个显示label）
            if (keys.size() > 2) {
                String label = keys.get(2);
                String type = keys.get(0);
                ComponentCons cons = new ComponentCons();
                cons.setLabel(label);
                cons.setVal(val);
                cons.setEl(item.variable());
                cons.setUploadFileUrl(uploadUrl);

                if (type.equalsIgnoreCase(ComponentEnum.DYNAMIC_TABLE.getValue())) {
                    HashSet<String> dhs = new HashSet<>(); //去除重复标签
                    List<TableColItem> colList = new ArrayList<>();
                    //表格 结构化绑定数据
                    ConfigureBuilder builderTable = Configure.builder();
                    builderTable.setValidErrorHandler(new Configure.DiscardHandler());
                    builderTable.useDefaultEL(false);
                    builderTable.buildGramer("[", "]");
                    XWPFTemplate templateTable = XWPFTemplate.compile(filepath, builderTable.build());

                    for (MetaTemplate col : templateTable.getElementTemplates()) {
                        if (dhs.contains(col.variable())) {
                            continue;
                        } else {
                            dhs.add(col.variable());
                        }
                        String colVal = StrUtil.strip(col.variable(), "[", "]");
                        if (!StrUtil.startWith(colVal, label)) {
                            continue;
                        }
                        List<String> colKeys = StrSplitter.split(colVal, "_", true, true);
                        //提取当前table中的列元素
                        if (colKeys.size() > 3) {
                            String colLabel = colKeys.get(3);
                            String colType = colKeys.get(1);
                            TableColItem tableColItem = new TableColItem();
                            tableColItem.setLabel(colLabel);
                            tableColItem.setColType(colType);
                            tableColItem.setVal(colVal);
                            colList.add(tableColItem);
                            log.info("列元素" + col.variable());
                        }
                    }
                    cons.setColList(colList);
                    DesignComponent designComponent = DesignComponent.createType(type);
                    ListItem listItem = designComponent.buildDefaultComponent(cons);
                    collect.add(listItem);
                    continue;
                }
                DesignComponent designComponent = DesignComponent.createType(type);
                ListItem listItem = designComponent.buildDefaultComponent(cons);
                collect.add(listItem);
            }
        }
        root.setList(collect);
        entity.setFormContent(JSON.toJSONString(root));
    }

    @Override
    public String mergeFormToDoc(MatterDatumEntity docFormVo, Context context) {
        String rootPath = this.filePath.endsWith("/") ? this.filePath : this.filePath + "/";
        //转换表单参数为map集合
        ObjectMapper mapper = new ObjectMapper();
        ConfigureBuilder builder = Configure.builder();
        builder.buildGrammerRegex(RegexUtils.createGeneral("{{", "}}"));
        try {
            Map<String, Object> data = mapper.readValue(docFormVo.getFormContent(), Map.class);
            Map<String, Object> addMap = new HashMap<>();
            //遍历查看是否有图片数据，多选框
            data.entrySet().stream().forEach(entry -> {
                if (entry.getKey().indexOf("@image") != -1) {
                    //获取值 读取本地图片
                    String imagepath = uploadService.getFilePath(entry.getValue().toString());
                    log.info("Image path: " + imagepath);
                    try {
                        PictureRenderData pictureRenderData = Pictures.ofStream(new FileInputStream(imagepath), PictureType.JPEG)
                                .size(100, 120).create();
                        addMap.put(StrUtil.removePrefixIgnoreCase(entry.getKey(), "@"), pictureRenderData);
                        //entry.setValue(pictureRenderData);
                    } catch (FileNotFoundException e) {
                        log.error("error", e);
                    }
                }
                //构建填充复选框值
                if (entry.getKey().indexOf("ck_") != -1 && entry.getKey().indexOf("<") != -1) {
                    //获取复选框选项，并渲染值
                    String ckStr = StringUtils.substringBetween(entry.getKey(), "<", ">");
                    //获取所有选项
                    List<String> checkboxs = StrUtil.splitTrim(ckStr, "-", -1);
                    //选中的项
                    List<String> ckList = Convert.toList(String.class, entry.getValue());
                    //合并
                    String ckStrs = checkboxs.stream().map(item -> {
                        for (String checked : ckList) {
                            if (checked.equals(item)) {
                                Map<Boolean, String> map = new HashMap<>();
                                map.put(true, item);
                                return map;
                            }
                        }
                        Map<Boolean, String> map = new HashMap<>();
                        map.put(false, item);
                        return map;
                    }).map(m -> m.entrySet().stream().map(m1 -> ExportDocUtil.createCheckBoxStrOther(m1.getKey(), m1.getValue())).findFirst().orElseGet(() -> "")
                    ).collect(Collectors.joining(""));
                    entry.setValue(ckStrs);
                }
                //TODO 单选
                //绑定动态表单
                if (entry.getKey().indexOf("dt_") != -1) {
                    builder.bind(entry.getKey(), new MultipleRowTableRenderPolicy());
                }
                //绑定转换list  list_1_身份证
                if (entry.getKey().indexOf("list_") != -1) {
                    String value = entry.getValue().toString();
                    SplitIter splitIter = new SplitIter(value, new LengthFinder(1), -1, false);
                    List<String> valueList = splitIter.toList(true);
                    builder.bind(entry.getKey(), new LoopSingleRowTableRenderPolicy());
                    data.put(entry.getKey(), valueList);
                }
            });
            if (addMap.size() > 0) {
                data.putAll(addMap);
            }
            //查询是否有多选框，
            //获取模板文件地址
            MatterDatumEntity docTemplateEntity = this.get(docFormVo.getId(), context);
            if (!ObjectUtils.isEmpty(docTemplateEntity)) {
                String path = rootPath + docTemplateEntity.getTemplatePath();
                String templateName = FileUtil.getName(path);
                Configure config = builder.build();
                XWPFTemplate template = null;
                ByteArrayOutputStream byteArrayOutputStream = null;
                try {
                    template = XWPFTemplate.compile(path, config).render(data);
                    byteArrayOutputStream = new ByteArrayOutputStream();
                    template.write(byteArrayOutputStream);

                    MultipartFile multipartFile = new MockMultipartFile(templateName, templateName,
                            ContentType.APPLICATION_OCTET_STREAM.toString(), byteArrayOutputStream.toByteArray());
                    String mergedocPath = uploadService.saveFileUpload(multipartFile, "mergedoc", context.getUser());
                    String mergedoc = rootPath + mergedocPath;
                    //转换预览图片
                    String fileName = RandomUtil.randomNumbers(12) + ".jpg";
                    String preView = rootPath + "file/preview/" + fileName;
                    log.info(String.format("mergedoc:%s,preViewPath:%s", mergedoc, preView));
                    WordUtil.convertWordToJPEG(mergedoc, preView);
                    log.info("preView:" + mergedocPath);
                    //下载地址拼装
                    String returnStr = mergedocPath + ";" + "file/preview/" + fileName;
                    return returnStr;
                } catch (Exception e) {
                    log.error("渲染模板失败：", e);
                } finally {
                    try {
                        if (template != null) {
                            template.close();
                        }
                        if (byteArrayOutputStream != null) {
                            byteArrayOutputStream.close();
                        }
                    } catch (IOException e) {
                        log.error("ioEx：", e);
                    }
                }
            }
        } catch (JsonProcessingException e) {
            log.error("转换异常", e);
            throw new AppException("表单参数转换异常:" + e.getMessage());
        }
        return "";
    }

    @Override
    public Rest<String> addPubdatum(Long[] pubIds, Long matterId, Long siteId, Context context) {
        if (ObjectUtils.isEmpty(pubIds)) {
            throw new AppException("公共材料库ids is null");
        }
        int success = 0;
        int fail = 0;
        List<PubdatumEntity> pubdatumEntityList = pubdatumService.get(pubIds, context);
        List<MatterDatumEntity> matterDatumEntities = new ArrayList<>();
        for (PubdatumEntity pubdatumEntity : pubdatumEntityList) {
            MatterDatumEntity matterDatumEntity = updateOrSave(pubdatumEntity, matterId, siteId, context);
            if (!ObjectUtils.isEmpty(matterDatumEntity)) {
                matterDatumEntities.add(matterDatumEntity);
                success++;
            } else {
                fail++;
            }
        }
        if (!ObjectUtils.isEmpty(matterDatumEntities)) {
            this.getDao().insertBatch(matterDatumEntities);
            // this.save(matterDatumEntities, context);
        }
        String msg = "当前加入材料已存在！";
        if (pubIds.length == 1) {
            if (success > 0) {
                msg = "加入材料成功！";
            }
        } else if (pubIds.length > 1) {
            if (success > 0 && fail == 0) {
                msg = String.format("加入材料成功%d条!", success);
            } else if (success > 0 && fail > 0) {
                msg = String.format("加入材料成功%d条，重复加入材料%d条！", success, fail);
            } else if (success == 0 && fail > 0) {
                msg = String.format("重复加入材料%d条！", fail);
            }
        }
        return Rest.ok(msg);
    }

    private MatterDatumEntity updateOrSave(PubdatumEntity pubdatumEntity, Long matterId, Long siteId, Context context) {
        int count = this.count(new MatterDatumQuery().siteId(siteId).matterId(matterId).materialName(pubdatumEntity.getMaterialName()), context);
        if (count == 0) {
            MatterDatumEntity entity = new MatterDatumEntity();
            entity.initAttrValue();
            BeanUtils.copyProperties(pubdatumEntity, entity, new String[]{"id", "matterId"});
            entity.setSiteId(siteId);
            entity.setMatterId(matterId);
            return entity;
        }
        return null;
    }

    @Override
    public Rest<String> recommend(Long id, Long siteId, Context context) {
        MatterDatumEntity matterDatumEntity = this.get(id, context);
        if (matterDatumEntity == null) {
            throw new AppException("数据已更改,请重试");
        }
        String msg = "推荐成功！";
        List<MatterEntity> matterEntities = matterService.find(new MatterQuery().siteId(siteId), context);
        int recommendCount = 0;
        for (MatterEntity matterEntity : matterEntities) {
            recommendCount += this.count(new MatterDatumQuery().matterId(matterEntity.getId()).isRecommend(YesNoEnum.YES.getValue()), context);
        }
        if (matterDatumEntity.getIsRecommend() == YesNoEnum.NO.getValue()) {
            if (recommendCount >= GlobalSysInfo.getParamIntValue(Constant.PARAMS_RECOMMEND_COUNT, 5)) {
                throw new AppException("超过推荐个数!");
            }
            matterDatumEntity.setIsRecommend(YesNoEnum.YES.getValue());
        } else {
            matterDatumEntity.setIsRecommend(YesNoEnum.NO.getValue());
            msg = "取消推荐成功！";
        }
        this.update(matterDatumEntity, context);
        return Rest.ok(msg);
    }


    @Override
    protected void findAfter(MatterDatumEntity params, PageInfo pageInfo, Context context, List<MatterDatumEntity> list) throws AppException {
        list.stream().forEach(entity -> {
            MatterEntity matterEntity = matterService.get(entity.getMatterId(), context);
            if (!ObjectUtils.isEmpty(matterEntity)) {
                entity.setMatterNo(matterEntity.getMatterNo());
                entity.setMatterName(matterEntity.getMatterName());
            }
        });
        super.findAfter(params, pageInfo, context, list);
    }

    public static void main(String[] args) {
        String colVal = "i_1_变更登记事项";
        List<String> colKeys = StrSplitter.split(colVal, "_", true, true);

        System.out.println(colKeys.size());
        System.out.println(colKeys.get(0));

        colVal = "13808095770";
        SplitIter splitIter = new SplitIter(colVal, new LengthFinder(1), -1, false);

        List<String> strings1 = splitIter.toList(true);
        strings1.forEach(item -> {
            System.out.println(item);
        });

//        String[] strings = StrSplitter.splitByLength(colVal, 1);
//        for (int i = 0; i <strings.length ; i++) {
//            System.out.println(strings[i]);
//        }
    }

}