package com.mortals.xhx.module.matter.web;

import cn.hutool.core.util.StrUtil;
import com.mortals.framework.common.IBaseEnum;
import com.mortals.framework.common.Rest;
import com.mortals.framework.common.code.YesNo;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.OrderCol;
import com.mortals.framework.service.IUser;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.code.MatterSourceEnum;
import com.mortals.xhx.common.key.ParamKey;
import com.mortals.xhx.common.utils.StringUtils;
import com.mortals.xhx.module.hotword.model.HotwordEntity;
import com.mortals.xhx.module.hotword.model.HotwordQuery;
import com.mortals.xhx.module.hotword.service.HotwordService;
import com.mortals.xhx.module.matter.model.MatterDatumQuery;
import com.mortals.xhx.module.matter.service.MatterDatumService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;
import com.mortals.framework.model.Context;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.module.matter.model.MatterEntity;
import com.mortals.xhx.module.matter.service.MatterService;
import org.apache.commons.lang3.ArrayUtils;

import java.util.*;
import java.util.stream.Collectors;

import com.alibaba.fastjson.JSONObject;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import static com.mortals.framework.ap.SysConstains.*;

/**
 * 事项申请材料
 *
 * @author zxfei
 * @date 2022-09-27
 */
@RestController
@RequestMapping("matter")
public class MatterController extends BaseCRUDJsonBodyMappingController<MatterService, MatterEntity, Long> {

    @Autowired
    private ParamService paramService;
    @Autowired
    private HotwordService hotwordService;

    @Autowired
    private MatterDatumService matterDatumService;

    public MatterController() {
        super.setModuleDesc("事项");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "isRecommend", paramService.getParamBySecondOrganize("Matter", "isRecommend"));
        this.addDict(model, "source", IBaseEnum.getEnumMap(MatterSourceEnum.class));
        super.init(model, context);
    }

    @Override
    protected void doListBefore(MatterEntity query, Map<String, Object> model, Context context) throws AppException {
        if (StringUtils.isNotEmpty(query.getMatterName())) {
            query.setMatterName("%".concat(query.getMatterName()).concat("%"));
        }

        if (StringUtils.isNotEmpty(query.getMatterFullName())) {
            query.setMatterFullName("%".concat(query.getMatterFullName()).concat("%"));
        }
        List<OrderCol> orderColList = new ArrayList<>();
        orderColList.add(new OrderCol("isRecommend", OrderCol.DESCENDING));
        orderColList.add(new OrderCol("createTime", OrderCol.DESCENDING));
        query.setOrderColList(orderColList);

        if ("%%".equals(query.getMatterFullName()) || "%null%".equals(query.getMatterFullName())) {
            return;
        }
        //添加更新热门词汇
        HotwordEntity hotwordEntity = hotwordService.selectOne(new HotwordQuery().siteId(query.getSiteId()).hotwords(query.getMatterFullName()), context);
        if (ObjectUtils.isEmpty(hotwordEntity)) {
            //新增
            hotwordEntity = new HotwordEntity();
            hotwordEntity.initAttrValue();
            hotwordEntity.setHotwords(StrUtil.subBetween(query.getMatterFullName(), "%"));
            hotwordEntity.setSearchCount(1);
            hotwordEntity.setSiteId(query.getSiteId());
            hotwordService.save(hotwordEntity, context);
        } else {
            //更新
            hotwordEntity.setSearchCount(1 + hotwordEntity.getSearchCount());
            hotwordService.update(hotwordEntity, context);
        }
        super.doListBefore(query, model, context);
    }

    @Override
    protected int doListAfter(MatterEntity query, Map<String, Object> model, Context context) throws AppException {
        //统计材料数量
        int count = 0;
        List<Long> matterIdList = this.service.find(query, context).stream().map(MatterEntity::getId).collect(Collectors.toList());
        MatterDatumQuery matterDatumQuery = new MatterDatumQuery();
        if (!ObjectUtils.isEmpty(matterIdList)) {
            count = matterDatumService.count(matterDatumQuery.siteId(query.getSiteId()), getContext());
        } else {
            matterDatumQuery.setMatterIdList(matterIdList);
        }
        model.put("matterDatumTotal", count);
        return super.doListAfter(query, model, context);
    }

    @RequestMapping(value = {"createMatter"}, method = {RequestMethod.POST, RequestMethod.GET})
    public String createMatter(Long[] ids, Long siteId) {
        Map<String, Object> model = new HashMap();
        int code = VALUE_RESULT_SUCCESS;
        String busiDesc = "站点事项转事项";

        try {
            Rest<String> rest = this.service.createMatterbBySheetMatter(ids, siteId, getContext());
            model.put("message_info", rest.getMsg());
            this.recordSysLog(this.request, rest.getMsg());
        } catch (Exception var7) {
            code = VALUE_RESULT_FAILURE;
            this.doException(this.request, busiDesc, model, var7);
        }

        JSONObject ret = new JSONObject();
        ret.put("code", Integer.valueOf(code));
        ret.put("msg", model.remove("message_info"));
        ret.put("data", model);
        return ret.toJSONString();
    }


    @RequestMapping(value = {"recommend"}, method = {RequestMethod.POST, RequestMethod.GET})
    public String recommend(Long id) {
        Map<String, Object> model = new HashMap();
        int code = VALUE_RESULT_SUCCESS;
        String busiDesc = "推荐or取消推荐" + this.getModuleDesc();
        try {
            Rest<String> rest = this.service.recommend(id, getContext());
            model.put("message_info", rest.getMsg());
            this.recordSysLog(this.request, rest.getMsg());
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            this.doException(this.request, busiDesc, model, e);
        }

        JSONObject ret = new JSONObject();
        ret.put("code", code);
        ret.put("msg", model.remove("message_info"));
        ret.put("data", model);
        return ret.toJSONString();
    }
}